@@ -2,7 +2,7 @@
 # archiver.py
 #
 
-import os, sys, socket, re, time, smtplib, calendar
+import os, sys, socket, re, time, smtplib, calendar, StringIO, logging, logging.handlers
 from datetime import datetime, timedelta
 import email
 from email.Utils import parseaddr, formataddr, formatdate
@@ -206,6 +206,31 @@ def sendmail(sender, recipients, subject, text, attachements, smtp_host, smtp_po
     smtp=smtplib.SMTP(smtp_host, smtp_port)
     ret=smtp.sendmail(sender, recipients, body)
 
+
+# ---------------------------------------------------------------------------
+def send_mail_report(job, result, manager):
+    backup_status, status, attachements=result
+
+    msg_body=job['msg_header']+'\n'+status
+    subject='MKBACKUP BACKUP %s %s' % (backup_status, job['name'])
+    
+    attachements.append(('config.ini', job['config'], None, 'text', 'plain', job['config_encoding']))
+    attachements.append(('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'))
+    log_output=manager.stop_logging()
+    attachements.append(('logging.txt', None, log_output.encode('utf-8'), 'text', 'plain', 'utf-8'))                       
+    
+    for line in status.split('\r\n'):
+        if line:
+           manager.log.info('    %s', line)
+    
+    if job['mail']:
+        sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'])
+
+    if status=='OK':
+        return 0
+    else:
+        return 1
+
 #======================================================================
 #
 # Destinations
@@ -324,6 +349,48 @@ class Destinations:
             
         return None, None
 
+# ---------------------------------------------------------------------------
+
+class MyMemoryHandler(logging.handlers.MemoryHandler):
+
+     def shouldFlush(self, record):
+        """
+        Check for buffer full and drop oldest records.
+        """
+        if len(self.buffer) >= self.capacity:
+            self.buffer=self.buffer[-self.capacity:]
+            
+        return False
+   
+
+# ---------------------------------------------------------------------------
+class Manager:
+    
+    memory_log_capacity=2000
+
+    def __init__(self, hostname, log):
+        self.hostname=hostname
+        self.log=log
+        
+    def start_logging(self):
+        self.string_log=StringIO.StringIO()
+        self.stream_log=logging.StreamHandler(self.string_log)
+        self.stream_log.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
+        self.memory_log=MyMemoryHandler(self.memory_log_capacity, logging.INFO, self.stream_log)
+        self.log.addHandler(self.memory_log)
+
+    def stop_logging(self):
+        self.memory_log.flush()
+        self.stream_log.flush()
+
+        self.log.removeHandler(self.memory_log)
+        
+        self.memory_log.close()
+        self.stream_log.close()
+        log_output=self.string_log.getvalue()
+        self.string_log.close()
+        return log_output
+
 # ---------------------------------------------------------------------------
 class Archiver:
     
@@ -336,6 +403,5 @@ class Archiver:
     def __init__(self):
         self.night_shift=True
 
-
     
     # -----------------------------------------------------------------------
