@@ -16,6 +16,7 @@ import scp
 from archiver import *
 import cron
 
+GHETTO_VERSION='04/24/2010'
 
 # ---------------------------------------------------------------------------
 class MySCPClient(scp.SCPClient):
@@ -135,7 +136,8 @@ class Ghettovcb(Archiver):
         self.password=None
         self.local=None
         self.remote_temp=None
-        self.vm_list=None
+        self.vm_list=[]
+        self.vm_exclude=[]
         
         self.ghettovcb=None
         self.ghettovcb_bin=None
@@ -146,8 +148,8 @@ class Ghettovcb(Archiver):
         self.default_var=None
         
         self.destination=None
-        self.target=None
-        self.type=None
+        self.type='backup'
+        self.target=''
         
         self.scp_bin=None
         self.scp_args=None
@@ -158,7 +160,7 @@ class Ghettovcb(Archiver):
         """load important variables from VM configuration backup script""" 
         values=default.copy()
         for line in open(filename, 'r'):
-            for var in ('VM_BACKUP_VOLUME', 'ENABLE_COMPRESSION'):
+            for var in ('VM_BACKUP_VOLUME', 'ENABLE_COMPRESSION', 'LAST_MODIFIED_DATE', 'ENABLE_NON_PERSISTENT_NFS',):
                 if line.startswith('printUsage() {'):
                     break
                 if line.startswith('%s=' % var):
@@ -168,7 +170,11 @@ class Ghettovcb(Archiver):
         return values
     
     # -----------------------------------------------------------------------
-    def load(self, job, log, now):
+    def load(self, job, manager):
+        
+        log=manager.log
+        now=manager.now
+        
         errors, warnings, extra={}, {}, ''
 
         try:
@@ -196,7 +202,7 @@ class Ghettovcb(Archiver):
             if not (0<self.port and self.port<65535):
                 errors['port']='must be an integer between 1 and 65535'
 
-        for name in ('login', 'password', 'local', 'remote_temp', 'vm_list',):
+        for name in ('login', 'password', 'local', 'remote_temp',):
             value=job.get(name, None)
             if not value:
                 errors[name]='option mandatory'
@@ -204,10 +210,15 @@ class Ghettovcb(Archiver):
                 setattr(self, name, value)
 
         try:
-            self.vm_list=quoted_string_list(self.vm_list)
+            self.vm_list=quoted_string_list(job.get('vm_list', ''))
         except ValueError:
             errors['vm_list']='not a valid quoted string list'
 
+        try:
+            self.vm_exclude=quoted_string_list(job.get('vm_exclude', ''))
+        except ValueError:
+            errors['vm_exclude']='not a valid quoted string list'
+
         if job.get('target', None):
             self.target=job['target']
 
@@ -248,6 +259,12 @@ class Ghettovcb(Archiver):
             self.default_var={}
             if self.ghettovcb:
                 self.default_var=self.read_vm_conf(os.path.join(self.local, self.ghettovcb))
+                
+                
+            if self.default_var.get('LAST_MODIFIED_DATE','')!=GHETTO_VERSION:
+                warning['local']='ghettoVCB.sh version is different, use the one provided with MKBackup or be warned'
+            
+            
             # read and extract gloabl_conf
             if self.global_conf:
                 self.default_var=self.read_vm_conf(os.path.join(self.local, self.global_conf), self.default_var)
@@ -291,13 +308,26 @@ class Ghettovcb(Archiver):
             scpclient=MySCPClient(t)
             scpclient.put(self.local, self.remote_temp, recursive=True)
 
-            self.temp_target=posixpath.join(self.remote_temp, posixpath.basename(self.local))
+            self.temp_target=posixpath.join(self.remote_temp, os.path.basename(self.local))
             self.ghettovcb_bin=posixpath.join(self.temp_target, self.ghettovcb)
             self.current_vm_file=posixpath.join(self.temp_target, 'current_vm')
+            self.exclude_vm_file=posixpath.join(self.temp_target, 'exclude_vm')
             
             _exit_code, _output=RunCommand(t, 'chmod +x "%s"' % self.ghettovcb_bin, log)
 
-            self.cmd_line='cd "%s" ; ./"%s" -d debug -f "%s" -c "%s"' % (self.temp_target, self.ghettovcb, self.current_vm_file, self.temp_target)
+            self.cmd_line='cd "%s" ; ./"%s" -d debug -c "%s"' % (self.temp_target, self.ghettovcb, self.temp_target)
+            
+            if self.vm_list:
+                self.cmd_line+=' -f "%s"' % (self.current_vm_file,)       
+            else:
+                self.cmd_line+=' -a'       
+                if self.vm_exclude:
+                    self.cmd_line+=' -e "%s"' % (self.exclude_vm_file,)       
+                    for i, vm in enumerate(self.vm_exclude):
+                        if i==0:
+                            _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (vm, self.exclude_vm_file), log)
+                        else:
+                            _exit_code, _output=RunCommand(t, 'echo "%s" >> "%s"' % (vm, self.exclude_vm_file), log)
             if self.global_conf:
                 self.cmd_line+=' -g "%s"' % (self.global_conf,)       
 
@@ -314,27 +344,47 @@ class Ghettovcb(Archiver):
         return errors, warnings, extra
 
     # -----------------------------------------------------------------------
-    def run(self, command, job, log, now):
+    def run(self, command, job, manager):
 
         status=0
-        for self.current_vm in self.vm_list:
-            
-            if self.sub_run(command, job, log, now)!=0:
-                status=1
+        name=job['name']
+        
+        if self.vm_list:
+            vm_list=self.vm_list
+        else:
+            vm_list=[ None, ]
+                
+        for self.current_vm in vm_list:
+            if self.current_vm:
+                job['name']='%s.%s' % (name, self.current_vm)
+                
+            result=self.sub_run(command, job, manager)
+
+            code=send_mail_report(job, result, manager)
+            status=max(status, code)
+
+        job['name']=name
 
         return status
         
     # -----------------------------------------------------------------------
-    def sub_run(self, command, job, log, now):
+    def sub_run(self, command, job, manager):
+
+        log=manager.log
+        now=manager.now
 
         start=int(time.time())
 
         t=paramiko.Transport((self.host, self.port))
         t.get_security_options()._set_ciphers(['blowfish-cbc', ])
         t.connect(username=self.login, password=self.password)
-        _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (self.current_vm, self.current_vm_file), log)
+        t.set_keepalive(300)
+        if self.current_vm:
+            # create the current_vm_file
+            _exit_code, _output=RunCommand(t, 'echo "%s" > "%s"' % (self.current_vm, self.current_vm_file), log)
         chan=t.open_session()
         chan.set_combine_stderr(True)
+        log.info('start: %s', self.cmd_line)
         chan.exec_command(self.cmd_line)
         output, line_buf='', ''
         while True:
@@ -356,68 +406,78 @@ class Ghettovcb(Archiver):
         end=int(time.time())
 
         # search the target and make a directory listing 
-        target=posixpath.join(self.vm_var[self.current_vm]['VM_BACKUP_VOLUME'], self.current_vm)
-        compress=self.vm_var[self.current_vm]['ENABLE_COMPRESSION']=='1'
-        posixpath.join(target, self.current_vm)
-        dir_lst=None
-        if compress:
-            _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
-        else:
-            _exit_code, up_lst=RunCommand(t, 'ls "%s"' % target, log)
-            dir_lst=None
-            # search the target directory, take the last one
-            target_name=''
-            for line in up_lst.split('\n'):
-                if line.startswith(self.current_vm):
-                    if line>target_name:
-                       target_name=line
-            if target_name: 
-                target=posixpath.join(target, target_name)
+        dir_lst, df, scp_process, target=None, None, None, None
+        if self.current_vm and self.vm_var[self.current_vm]['ENABLE_NON_PERSISTENT_NFS']!=1:
+            target=posixpath.join(self.vm_var[self.current_vm]['VM_BACKUP_VOLUME'], self.current_vm)
+            compress=self.vm_var[self.current_vm]['ENABLE_COMPRESSION']=='1'
+            posixpath.join(target, self.current_vm)
+            if compress:
                 _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
-
-        # if destination, move or copy backup
-        scp_process=None
-        if self.destination:
-            self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.current_vm))
-            if self.type in ('move', 'copy'):
-                if not self.scp_bin:
-                    # use paramiko
-                    scpclient=MySCPClient(t, buff_size=65536)
-                    log.info('scp -r %s %s', target, self.target)
-                    scpclient.get(target, self.target, recursive=True)
-                else:
-                    # use local SCP
-                    scp_args=self.scp_args[:]
+            else:
+                _exit_code, up_lst=RunCommand(t, 'ls "%s"' % target, log)
+                dir_lst=None
+                # search the target directory, take the last one
+                target_name=''
+                for line in up_lst.split('\n'):
+                    if line.startswith(self.current_vm):
+                        if line>target_name:
+                           target_name=line
+                if target_name: 
+                    target=posixpath.join(target, target_name)
+                    _exit_code, dir_lst=RunCommand(t, 'ls -l "%s"' % target, log)
+    
+            # if destination, move or copy backup
+            if self.destination:
+                self.type, self.target=self.destination.match(now, self.night_shift, variables=dict(vm=self.current_vm))
+                if self.type in ('move', 'copy'):
+                    
                     if not compress:
-                        if sys.platform in ('win32', ):
-                            scp_args.append('-unsafe')
-                        #scp_args.append('-r')
-                        wildcard='/*'
+                        # create the target dir, if dont exist
+                        try:
+                            os.mkdir(self.target)
+                        except OSError, e:
+                            log.debug('creating directory %s: %s', self.target, e)
+                        except:
+                            log.error('creating directory %s: %s', self.target, e)
+    
+                    if not self.scp_bin:
+                        # use paramiko
+                        scpclient=MySCPClient(t, buff_size=65536)
+                        log.info('scp -r %s %s', target, self.target)
+                        scpclient.get(target, self.target, recursive=True)
                     else:
-                        wildcard=''
+                        # use local SCP
+                        scp_args=self.scp_args[:]
+                        if not compress:
+                            if sys.platform in ('win32', ):
+                                scp_args.append('-unsafe')
+                            #scp_args.append('-r')
+                            wildcard='/*'
+                        else:
+                            wildcard=''
+                            
+                        scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
+                        scp_cmd=' '.join(scp_args)
+                        log.info('scp_cmd=%s', scp_cmd)
+                        mypass={ '<password>' : self.password}
+                        scp_args=map(lambda x: mypass.get(x, x), scp_args)
+                        scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+                        scp_out, scp_err=scp_process.communicate()
+    
+                    log.info('scp end')
                         
-                    scp_args+=[ '%s@%s:%s%s' % (self.login, self.host, target, wildcard), self.target]
-                    scp_cmd=' '.join(scp_args)
-                    log.info('scp_cmd=%s', scp_cmd)
-                    mypass={ '<password>' : self.password}
-                    scp_args=map(lambda x: mypass.get(x, x), scp_args)
-                    scp_process=subprocess.Popen(scp_args, executable=self.scp_bin, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                    scp_out, scp_err=scp_process.communicate()
-
-                log.info('scp end')
-                    
-                    
-            if self.type=='move':
-                # delete the source
-                log.info('delete %s', target)
-                _exit_code, dir_lst=RunCommand(t, 'rm -r "%s"' % target, log)
-
-        # make the disk space usage
-        _exit_code, df=RunCommand(t, 'df', log)
+                        
+                if self.type=='move':
+                    # delete the source
+                    log.info('delete %s', target)
+                    _exit_code, dir_lst=RunCommand(t, 'rm -r "%s"' % target, log)
+    
+            # make the disk space usage
+            _exit_code, df=RunCommand(t, 'df', log)
 
         t.close()
 
-        if ghettovcb_exit_code==0 and not scp_process or scp_process.returncode==0:
+        if ghettovcb_exit_code==0 and (not scp_process or scp_process.returncode==0):
             backup_status='OK'
         else:
             backup_status='ERR'
@@ -427,12 +487,13 @@ class Ghettovcb(Archiver):
         status+='program=%s\r\n' % self.name
         status+='version=%s\r\n' % self.__version__
         status+='status=%s\r\n' % backup_status
-        status+='hostname=%s\r\n' % platform.node()
+        status+='hostname=%s\r\n' % manager.hostname
         
-        status+='exit_code=%d\n' % ghettovcb_exit_code
+        status+='exit_code=%d\r\n' % ghettovcb_exit_code
         status+='start=%s\r\n' % time.ctime(start)
         status+='end=%s\r\n' % time.ctime(end)
-        status+='remote_target=%s\r\n' % target
+        if target:
+            status+='remote_target=%s\r\n' % target
 
         status+='type=%s\r\n' % self.type
         status+='target=%s\r\n' % self.target
@@ -442,35 +503,20 @@ class Ghettovcb(Archiver):
         status+='start_epoch=%d\r\n' % start
         status+='end_epoch=%d\r\n' % end
 
-        for line in status.split('\r\n'):
-            if line:
-                log.info('    %s', line)
-        
-        #
-        # dir of target directory
-        #
-
         attachements=[ # the type, subtype and coding is not used
-                       ('config.ini', job['config'], None, 'text', 'plain', job['config_encoding']),
-                       ('status.txt', None, status.encode('utf-8'), 'text', 'plain', 'utf-8'),
                        ('output.txt', None, output, 'text', 'plain', 'utf-8'),
-                       ('df.txt', None, df, 'text', 'plain', 'utf-8'),
                     ]
         if dir_lst:
             attachements.append(('dir.txt', None, dir_lst, 'text', 'plain', 'utf-8'))
-            
+
+        if df:
+            attachements.append(('df.txt', None, df, 'text', 'plain', 'utf-8'),)
+
         if scp_process and scp_out:
             attachements.append(('scp_out.txt', None, scp_out, 'text', 'plain', 'cp850'))
         if scp_process and scp_err:
             attachements.append(('scp_err.txt', None, scp_err, 'text', 'plain', 'cp850'))
 
         
-        msg_body=job['msg_header']+'\n'+status
-        subject='MKBACKUP BACKUP %s %s.%s' % (backup_status, job['name'], self.current_vm)
-        if job['mail']:
-            sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'])
+        return backup_status, status, attachements
 
-        if backup_status=='OK':
-            return 0
-        else:
-            return 1
