@@ -29,19 +29,14 @@ import ConfigParser
 
 from datetime import datetime, timedelta
 
-from archiver import boolean, check_mail_config, Destinations, sendmail
+from archiver import boolean, check_mail_config, Destinations, sendmail, Manager, send_mail_report
 
 import cron
 
-
-
-memory_log_capacity=2000
-__version__='0.8.1b'
+__version__='0.8.3b'
 last_version_url='http://www.magikmon.com/download/mkbackup_last.txt'
 
 
-
-
 def gen_archiver(name):
     name=name.lower()
     if name=='ntbackup':
@@ -120,19 +115,6 @@ def mail_exception(start, job, extra, attachements):
     tb.close()
     sendmail(job['sender'], job['recipients'], subject, msg_body, attachements, job['smtp_host'], job['smtp_port'])
 
-# ---------------------------------------------------------------------------
-
-class MyMemoryHandler(logging.handlers.MemoryHandler):
-
-     def shouldFlush(self, record):
-        """
-        Check for buffer full and drop oldest records.
-        """
-        if len(self.buffer) >= self.capacity:
-            self.buffer=self.buffer[-self.capacity:]
-            
-        return False
-   
 
 #======================================================================
 #
@@ -174,11 +156,6 @@ if cmd_options.verbose:
    console.setFormatter(logging.Formatter('%(asctime)s,%(msecs)03d %(levelname)-3.3s %(message)s', '%H:%M:%S'))
    log.addHandler(console)
 
-#string_log=StringIO.StringIO()
-#stream_log=logging.handlers.StreamHandler(string_log())
-#stream_log.setFormatter(logging.Formatter('%(asctime)s %(levelname)-3.3s %(message)s', '%Y-%m-%d %H:%M:%S'))
-#memory_log=MyMemoryHandler(memory_log_capacity, logging.INFO, stream_log)
-#log.addHandler(memory_log)
 
 log.info('start %r', sys.argv)
 
@@ -290,9 +267,14 @@ if err>0:
 exit_code=0
 
 
+manager=Manager(platform.node(), log)
+
 for job_name in job_list:
 
+    manager.start_logging()
+
     now=datetime.now()
+    manager.now=now
     start=int(time.time())
     job=dict(config.items(job_name))
     program=job.get('program')
@@ -316,7 +298,7 @@ for job_name in job_list:
     # check job parameters 
     
     try:
-        errors, warnings, extra=arch.load(job, log, now)
+        errors, warnings, extra=arch.load(job, manager)
     except Exception, e:
         log.exception('loading job %s by program %s', job_name, program)
         if job['mail'] and not mail_config_errors:
@@ -390,7 +372,7 @@ for job_name in job_list:
 
     log.info('start command=%s job=%s archiver=%s', command, job_name, program)
     try:
-        status=arch.run(command, job, log, now)
+        result=arch.run(command, job, manager)
     except Exception, e:
         exit_code=2
         log.exception('running command=%s job=%s archiver=%s', command, job_name, program)
@@ -398,9 +380,13 @@ for job_name in job_list:
             mail_exception(start, job, extra, attachements)
     else:
         log.info('end command=%s job=%s archiver=%s', command, job_name, program)
-        if status!=0:
-            exit_code=1
 
+        if isinstance(result, int):
+            exit_code=max(exit_code, result)
+        else:
+            code=send_mail_report(job, result, manager)
+            exit_code=max(exit_code, code)
+            
 
 sys.exit(exit_code)
 
